package com.spring.series.jdbc.dao.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.stereotype.Repository;

import com.spring.series.jdbc.dao.PlayerDAO;
import com.spring.series.jdbc.model.Player;
import com.spring.series.jdbc.utils.PlayerRowMapper;

@Repository("playerDAO")
public class PlayerDAOImpl implements PlayerDAO {

	// member variables
	@Autowired
	private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

	public NamedParameterJdbcTemplate getNamedParameterJdbcTemplate() {
		return namedParameterJdbcTemplate;
	}

	/**
	 * create or inserts the new player information into the database using simpleJdbcTemplate
	 */
	public String createOrSaveNewPLayer(Player player) {

		// local variables
		String sql = "INSERT INTO PLAYER(NAME, AGE, MATCHES) VALUES( :name, :age, :matches)";
		SqlParameterSource namedParameters = new MapSqlParameterSource()
		.addValue("name", player.getName())
		.addValue("age", player.getAge())
		.addValue("matches", player.getMatches());

		int returnValue = getNamedParameterJdbcTemplate().update(
				sql, 
				namedParameters);

		if(1 == returnValue)
			return "Player creation is SUCCESS";
		else
			return "Player creation is FAILURE";
	}

	/**
	 * This method retrieves a player from database using jdbcTemplate based on the PLAYER_ID supplied in the formal arguments
	 */
	public Player getPlayer(int playerId) {

		String sql = "SELECT PLAYER_ID, NAME, AGE, MATCHES FROM PLAYER WHERE PLAYER_ID = :player_id";
		SqlParameterSource namedParameters = new MapSqlParameterSource("player_id", playerId);
		Player player = getNamedParameterJdbcTemplate().queryForObject(
				sql, 
				namedParameters, 
				new PlayerRowMapper());
		return player;
	}

	/**
	 * This method updates the player information in the database using simpleJdbcTemplate
	 */
	public String updatePlayerInfo(Player player) {

		String sql = "UPDATE PLAYER SET NAME = :name, AGE = :age, MATCHES = :matches WHERE PLAYER_ID = :player_id";
		SqlParameterSource namedParameters = new MapSqlParameterSource()
		.addValue("name", player.getName())
		.addValue("age", player.getAge())
		.addValue("matches", player.getMatches())
		.addValue("player_id", player.getPlayerId());

		int returnValue = getNamedParameterJdbcTemplate().update(
				sql, 
				namedParameters);

		if(1 == returnValue)
			return "Player updation is SUCCESS";
		else
			return "Player updation is FAILURE";
	}

	/**
	 * This method deletes the player information from the database using simpleJdbcTemplate
	 */
	public String deletePlayerInfo(String player_id) {

		String sql = "DELETE FROM PLAYER WHERE PLAYER_ID =:playerId ";
		SqlParameterSource namedParameters = new MapSqlParameterSource("playerId", player_id);

		int returnValue = getNamedParameterJdbcTemplate().update(
				sql, 
				namedParameters);

		if(1 == returnValue)
			return "Player deletion is SUCCESS";
		else
			return "Player deletion is FAILURE";
	}

	/**
	 * Retrieves all players from the database using simpleJdbcTemplate
	 */
	public List<Player> getAllPlayer() {

		// local variables
		String sql = "SELECT PLAYER_ID, NAME, AGE, MATCHES FROM PLAYER";
		List<Player> lstPlayers  = getNamedParameterJdbcTemplate().query(
				sql, 
				new PlayerRowMapper());
		return lstPlayers;
	}
}